@@ -283,6 +283,21 @@ class DeploymentContext(EntityContext):
     def runtime_only_evaluation(self):
         return self._context.get('runtime_only_evaluation')
 
+    @property
+    def display_name(self):
+        """The deployment's display_name used e.g. in Cloudify UI."""
+        return self._context.get('deployment_display_name')
+
+    @property
+    def creator(self):
+        """The name of the user who owns the deployment."""
+        return self._context.get('deployment_creator')
+
+    @property
+    def resource_tags(self):
+        """The name of the user who owns the deployment."""
+        return self._context.get('deployment_resource_tags')
+
 
 class NodeContext(EntityContext):
 
@@ -327,6 +342,12 @@ class NodeContext(EntityContext):
         self._get_node_if_needed()
         return self._node.type_hierarchy
 
+    @property
+    def number_of_instances(self):
+        """The number of instances of that node."""
+        self._get_node_if_needed()
+        return self._node.number_of_instances
+
 
 class NodeInstanceContext(EntityContext):
     def __init__(self, *args, **kwargs):
@@ -337,6 +358,7 @@ class NodeInstanceContext(EntityContext):
         self._node_instance = None
         self._host_ip = None
         self._relationships = None
+        self._scaling_groups = []
 
     def _get_node_instance(self):
         self._node_instance = self._endpoint.get_node_instance(self.id)
@@ -368,6 +390,12 @@ class NodeInstanceContext(EntityContext):
         self._get_node_instance_if_needed()
         return self._node_instance.runtime_properties
 
+    @property
+    def scaling_groups(self):
+        """The list of scaling group instances this node instance belongs to"""
+        self._get_node_instance_if_needed()
+        return self._node_instance.scaling_groups
+
     @runtime_properties.setter
     def runtime_properties(self, new_properties):
         self._get_node_instance_if_needed()
@@ -1057,6 +1085,10 @@ class OperationContext(object):
         """The maximum number of retries the operation can have."""
         return self._operation_context.get('max_retries')
 
+    @property
+    def relationship(self):
+        return self._operation_context.get('relationship')
+
     def retry(self, message=None, retry_after=None):
         """Specifies that this operation should be retried.
 
