@@ -1,18 +1,3 @@
-########
-# Copyright (c) 2018 Cloudify Platform Ltd. All rights reserved
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-#    * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#    * See the License for the specific language governing permissions and
-#    * limitations under the License.
-
 import os
 import requests
 
@@ -39,7 +24,8 @@ class NodeInstance(object):
                  version=None,
                  host_id=None,
                  relationships=None,
-                 index=None):
+                 index=None,
+                 scaling_groups=None):
         self.id = node_instance_id
         self._node_id = node_id
         self._runtime_properties = \
@@ -49,6 +35,7 @@ class NodeInstance(object):
         self._host_id = host_id
         self._relationships = relationships
         self._index = index
+        self._scaling_groups = scaling_groups
 
     def get(self, key):
         return self._runtime_properties.get(key)
@@ -124,6 +111,10 @@ class NodeInstance(object):
     def index(self):
         return self._index
 
+    @property
+    def scaling_groups(self):
+        return {g['id']: g['name'] for g in self._scaling_groups}
+
 
 def get_rest_client(tenant=None, api_token=None):
     """
@@ -389,7 +380,8 @@ def get_node_instance(node_instance_id, evaluate_functions=False, client=None):
                         version=instance.version,
                         host_id=instance.host_id,
                         relationships=instance.relationships,
-                        index=instance.index)
+                        index=instance.index,
+                        scaling_groups=instance.scaling_groups)
 
 
 def update_node_instance(node_instance, client=None):
@@ -505,3 +497,8 @@ class DirtyTrackingDict(dict):
             raise NonRecoverableError('Cannot modify runtime properties of'
                                       ' relationship node instances')
         self.dirty = True
+
+
+def _get_workdir_path(deployment_id, tenant):
+    return os.path.join('/opt', 'manager', 'resources', 'deployments',
+                        tenant, deployment_id)
