@@ -18,32 +18,15 @@ import testtools
 
 from cloudify import dispatch
 from cloudify import exceptions
-from cloudify.workflows import tasks
 from cloudify_rest_client.exceptions import InvalidExecutionUpdateStatus
 
 
 class TestDispatchTaskHandler(testtools.TestCase):
-    @patch('cloudify.dispatch.amqp_client_utils')
-    def test_dispatch_update_operation_resume(self, _):
-        """When the operation was already started, we set resume=true
-
-        (and the function fails because it's not resumable)
-        """
-        args = [1]
-        op_handler = self._operation(
-            func1, task_target='stub', args=args)
-        with patch('cloudify.context.CloudifyContext.get_operation',
-                   return_value=Mock(state=tasks.TASK_STARTED)):
-            exc = self.assertRaises(exceptions.NonRecoverableError,
-                                    op_handler.handle)
-        self.assertIn('not resumable', str(exc))
-
     def test_dispatch_no_such_handler(self):
         context = {'type': 'unknown_type'}
         self.assertRaises(exceptions.NonRecoverableError,
                           dispatch.dispatch, context)
 
-    @patch('cloudify.dispatch.amqp_client_utils')
     @patch('cloudify.dispatch.get_rest_client')
     @patch('cloudify.dispatch.WorkflowHandler._workflow_cancelled')
     @patch('cloudify.dispatch.update_execution_status',
@@ -81,7 +64,6 @@ class TestDispatchTaskHandler(testtools.TestCase):
             workflow_handler._func = _normal_func
             workflow_handler._ctx = _normal_ctx
 
-    @patch('cloudify.dispatch.amqp_client_utils')
     @patch('cloudify.dispatch.get_rest_client')
     @patch('cloudify.dispatch.WorkflowHandler._workflow_cancelled')
     @patch('cloudify.dispatch.update_execution_status',
@@ -116,7 +98,6 @@ class TestDispatchTaskHandler(testtools.TestCase):
             workflow_handler._func = _normal_func
             workflow_handler._ctx = _normal_ctx
 
-    @patch('cloudify.dispatch.amqp_client_utils')
     @patch('cloudify.dispatch.get_rest_client')
     @patch('cloudify.dispatch.WorkflowHandler._workflow_cancelled')
     @patch('cloudify.dispatch.update_execution_status',
@@ -153,7 +134,6 @@ class TestDispatchTaskHandler(testtools.TestCase):
             workflow_handler._func = _normal_func
             workflow_handler._ctx = _normal_ctx
 
-    @patch('cloudify.dispatch.amqp_client_utils')
     @patch('cloudify.dispatch.get_rest_client')
     @patch('cloudify.dispatch.update_execution_status')
     def test_workflow_update_execution_status_set_to_false(
