@@ -215,6 +215,8 @@ class TaskDependencyGraph(object):
         if stored_graph:
             self.id = stored_graph['id']
             self._stored = True
+            for task in self._tasks.values():
+                task.stored = True
 
     @property
     def tasks(self):
@@ -348,7 +350,7 @@ class TaskDependencyGraph(object):
             if not self._waiting_for:
                 return True
             deadline = self._error_time + self.ctx.wait_after_fail
-            if deadline > time.time():
+            if time.time() > deadline:
                 return True
             else:
                 self._wake_after_fail = threading.Timer(
@@ -373,20 +375,16 @@ class TaskDependencyGraph(object):
         if handler_result.action == tasks.HandlerResult.HANDLER_FAIL:
             if isinstance(task, SubgraphTask) and task.failed_task:
                 task = task.failed_task
-            message = "Task failed '{0}'".format(task.name)
-            if result and not isinstance(result, tasks.HandlerResult):
-                message = '{0} -> {1}'.format(message, result)
-            if self._error is None:
-                self._error = WorkflowFailed(message)
-                self._error_time = time.time()
+            result = self._task_error(result, task)
         elif handler_result.action == tasks.HandlerResult.HANDLER_RETRY:
             new_task = handler_result.retried_task
             if self.id is not None:
-                self.ctx.store_operation(
+                stored = self.ctx.store_operation(
                     new_task,
                     [dep.id for dep in self._dependencies[task]],
                     self.id)
-                new_task.stored = True
+                if stored:
+                    new_task.stored = True
             self.add_task(new_task)
             for dependency in self._dependencies[task]:
                 self.add_dependency(new_task, self.get_task(dependency))
@@ -396,6 +394,23 @@ class TaskDependencyGraph(object):
         self.remove_task(task)
         self._tasks_wait.set()
 
+    def _task_error(self, result, task):
+        message = "Task failed '{0}'".format(task.name)
+        causes_text = _task_error_causes_text(task)
+        if causes_text:
+            message = '{0}: {1}'.format(message, causes_text)
+        if result and not isinstance(result, tasks.HandlerResult):
+            result = str(result)
+            if result:
+                message = '{0} -> {1}'.format(message, result)
+        if self._error is None:
+            self._error = WorkflowFailed(message)
+            self._error_time = time.time()
+            self._waiting_for = {
+                t for t in self._waiting_for if not t.is_subgraph
+            }
+        return result
+
 
 class forkjoin(object):
     """
@@ -554,3 +569,16 @@ class SubgraphTask(tasks.WorkflowTask):
 
 def _on_failure_handler_fail(task):
     return tasks.HandlerResult.fail()
+
+
+def _task_error_causes_text(task):
+    if not hasattr(task, 'error') or not isinstance(task.error, dict):
+        return ''
+    return "\n".join(
+        "{0} `{1}`\n{2}".format(
+            c['type'],
+            c['message'],
+            c.get('traceback').strip() if c.get('traceback') else ''
+        ) for c in
+        task.error.get('known_exception_type_kwargs', {}).get('causes')
+    )
