@@ -24,8 +24,10 @@ from requests.packages import urllib3
 from cloudify import constants
 from cloudify.utils import ipv6_url_compat
 
+from ._compat import PY36PLUS
 from .utils import is_kerberos_env
 from cloudify_rest_client import exceptions
+from cloudify_rest_client.idp import IdentityProviderClient
 from cloudify_rest_client.ldap import LdapClient
 from cloudify_rest_client.nodes import NodesClient
 from cloudify_rest_client.users import UsersClient
@@ -68,6 +70,7 @@ from cloudify_rest_client.labels import (DeploymentsLabelsClient,
 from cloudify_rest_client.filters import (DeploymentsFiltersClient,
                                           BlueprintsFiltersClient)
 from cloudify_rest_client.workflows import WorkflowsClient
+from cloudify_rest_client.audit_log import AuditLogClient
 
 try:
     from requests_kerberos import HTTPKerberosAuth
@@ -77,6 +80,11 @@ except Exception:
     # Kerberos users will need to manually install it.
     HTTPKerberosAuth = None
 
+if PY36PLUS:
+    from cloudify_async_client.audit_log import AuditLogAsyncClient
+else:
+    AuditLogAsyncClient = None
+
 DEFAULT_PORT = 80
 SECURED_PORT = 443
 SECURED_PROTOCOL = 'https'
@@ -155,7 +163,8 @@ class HTTPClient(object):
                 error_msg,
                 status_code=response.status_code,
                 response=response)
-        message = result['message']
+        # this can be changed after RD-3539
+        message = result.get('message') or result.get('detail')
         code = result.get('error_code')
         server_traceback = result.get('server_traceback')
         self._prepare_and_raise_exception(
@@ -462,6 +471,7 @@ class CloudifyClient(object):
                                          token, tenant, kerberos_env, timeout,
                                          session)
         self.blueprints = BlueprintsClient(self._client)
+        self.idp = IdentityProviderClient(self._client)
         self.permissions = PermissionsClient(self._client)
         self.snapshots = SnapshotsClient(self._client)
         self.deployments = DeploymentsClient(self._client)
@@ -501,3 +511,7 @@ class CloudifyClient(object):
         self.deployments_labels = DeploymentsLabelsClient(self._client)
         self.blueprints_labels = BlueprintsLabelsClient(self._client)
         self.workflows = WorkflowsClient(self._client)
+        if AuditLogAsyncClient is None:
+            self.auditlog = AuditLogClient(self._client)
+        else:
+            self.auditlog = AuditLogAsyncClient(self._client)
