@@ -95,7 +95,8 @@ class ExecutionSchedulesClient(object):
     def create(self, schedule_id, deployment_id, workflow_id,
                execution_arguments=None, parameters=None,
                since=None, until=None, recurrence=None, count=None,
-               weekdays=None, rrule=None, slip=0, stop_on_fail=False):
+               weekdays=None, rrule=None, slip=0, stop_on_fail=False,
+               creator=None, created_at=None):
         """Schedules a deployment's workflow execution whose id is provided.
 
         :param schedule_id: Name for the schedule task. Used for listing,
@@ -131,6 +132,8 @@ class ExecutionSchedulesClient(object):
             in which the scheduled execution can run (in minutes).
         :param stop_on_fail: If set to true, once the execution has failed,
             the scheduler won't make further attempts to run it.
+        :param creator: Override the creator. Internal use only.
+        :param created_at: Override the creation timestamp. Internal use only.
         :return: The created execution schedule.
         """
         assert schedule_id
@@ -151,6 +154,10 @@ class ExecutionSchedulesClient(object):
             'slip': slip,
             'stop_on_fail': stop_on_fail
         }
+        if creator:
+            data['creator'] = creator
+        if created_at:
+            data['created_at'] = created_at
         uri = '/{self._uri_prefix}/{id}'.format(self=self, id=schedule_id)
         response = self.api.put(uri,
                                 data=data,
@@ -160,7 +167,7 @@ class ExecutionSchedulesClient(object):
 
     def update(self, schedule_id, deployment_id, since=None, until=None,
                recurrence=None, count=None, weekdays=None, rrule=None,
-               slip=None, stop_on_fail=None, enabled=None):
+               slip=None, stop_on_fail=None, enabled=None, workflow_id=None):
         """Updates scheduling parameters of an existing execution schedule
         whose id is provided.
 
@@ -189,6 +196,7 @@ class ExecutionSchedulesClient(object):
             the scheduler won't make further attempts to run it.
         :param enabled: Set to false to make the scheduler ignore this
             schedule, until set to true again.
+        :param workflow_id: The new workflow that this schedule is going to run
 
         :return: The updated execution schedule.
         """
@@ -203,7 +211,8 @@ class ExecutionSchedulesClient(object):
             'rrule': rrule,
             'slip': slip,
             'enabled': enabled,
-            'stop_on_fail': stop_on_fail
+            'stop_on_fail': stop_on_fail,
+            'workflow_id': workflow_id,
         }
         uri = '/{self._uri_prefix}/{id}'.format(self=self, id=schedule_id)
         response = self.api.patch(uri,
